@@ -1,9 +1,10 @@
 __all__ = (
+    "DiffsFoundError",
     "EyesError",
+    "NewTestError",
     "OutOfBoundsError",
     "TestFailedError",
-    "NewTestError",
-    "DiffsFoundError",
+    "USDKFailure",
 )
 
 
@@ -54,3 +55,9 @@ class DiffsFoundError(TestFailedError):
     """
     Indicates that an existing test ended, and that differences where found from the baseline.
     """
+
+
+class USDKFailure(EyesError):
+    """
+    Generic error raised by Universal SDK
+    """
