@@ -3,8 +3,6 @@ from __future__ import unicode_literals
 from typing import TYPE_CHECKING, List, Optional, Text, Tuple, Union, overload
 
 import attr
-from selenium.webdriver.common.by import By
-from selenium.webdriver.remote.webelement import WebElement
 from six import string_types
 
 from applitools.common.accessibility import AccessibilityRegionType
@@ -16,14 +14,11 @@ from applitools.common.validators import is_list_or_tuple, is_webelement
 from applitools.core.fluent import CheckSettings, CheckSettingsValues
 
 from .region import (
-    AccessibilityRegionByElement,
     AccessibilityRegionBySelector,
-    FloatingRegionByElement,
     FloatingRegionBySelector,
-    RegionByElement,
     RegionBySelector,
 )
-from .target_path import Locator, TargetPath
+from .target_path import RegionLocator, TargetPath
 
 if TYPE_CHECKING:
     from applitools.common.utils.custom_types import (
@@ -42,12 +37,10 @@ BEFORE_CAPTURE_SCREENSHOT = "beforeCaptureScreenshot"
 
 @attr.s
 class FrameLocator(object):
-    frame_element = attr.ib(default=None)  # type: AnyWebElement
-    frame_selector = attr.ib(default=None)  # type: BySelector
+    frame_locator = attr.ib(default=None)  # type: RegionLocator
     frame_name_or_id = attr.ib(default=None)  # type: FrameNameOrId
     frame_index = attr.ib(default=None)  # type: FrameIndex
-    scroll_root_selector = attr.ib(default=None)  # type: BySelector
-    scroll_root_element = attr.ib(default=None)  # type: AnyWebElement
+    scroll_root_locator = attr.ib(default=None)  # type: RegionLocator
 
 
 @attr.s
@@ -60,18 +53,12 @@ class LazyLoadOptions(object):
 @attr.s
 class SeleniumCheckSettingsValues(CheckSettingsValues):
     # hide_caret = attr.ib(init=False, default=None)
-    scroll_root_element = attr.ib(
+    scroll_root_locator = attr.ib(
         metadata={JsonInclude.NON_NONE: True}, init=False, default=None
-    )  # type: WebElement
-    scroll_root_selector = attr.ib(
+    )  # type: RegionLocator
+    target_locator = attr.ib(
         metadata={JsonInclude.NON_NONE: True}, init=False, default=None
-    )  # type: Locator
-    target_selector = attr.ib(
-        metadata={JsonInclude.NON_NONE: True}, init=False, default=None
-    )  # type: Locator
-    target_element = attr.ib(
-        metadata={JsonInclude.NON_NONE: True}, init=False, default=None
-    )  # type: WebElement
+    )  # type: RegionLocator
     frame_chain = attr.ib(
         metadata={JsonInclude.NON_NONE: True}, init=False, factory=list
     )  # type: List[FrameLocator]
@@ -110,11 +97,7 @@ class SeleniumCheckSettingsValues(CheckSettingsValues):
     @property
     def is_target_empty(self):
         # type: () -> bool
-        return (
-            self.target_region is None
-            and self.target_selector is None
-            and self.target_element is None
-        )
+        return self.target_region is None and self.target_locator is None
 
 
 @attr.s
@@ -143,7 +126,7 @@ class SeleniumCheckSettings(CheckSettings):
 
     @overload  # noqa
     def layout(self, *target_path, **kwargs):
-        # type: (*Locator,**Optional[CodedRegionPadding]) -> SeleniumCheckSettings
+        # type: (*RegionLocator,**Optional[CodedRegionPadding]) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
@@ -179,7 +162,7 @@ class SeleniumCheckSettings(CheckSettings):
 
     @overload  # noqa
     def strict(self, *target_path, **kwargs):
-        # type: (*Locator,**Optional[CodedRegionPadding]) -> SeleniumCheckSettings
+        # type: (*RegionLocator,**Optional[CodedRegionPadding]) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
@@ -214,8 +197,12 @@ class SeleniumCheckSettings(CheckSettings):
         pass
 
     @overload  # noqa
-    def content(self, *target_path, **kwargs):
-        # type: (*Locator, **Optional[CodedRegionPadding]) -> SeleniumCheckSettings
+    def content(
+        self,
+        *target_path,  # type: RegionLocator
+        **kwargs  # type: Optional[CodedRegionPadding]
+    ):
+        # type: (...) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
@@ -250,8 +237,12 @@ class SeleniumCheckSettings(CheckSettings):
         pass
 
     @overload  # noqa
-    def ignore(self, *target_path, **kwargs):
-        # type: (*Locator, **Optional[CodedRegionPadding]) -> SeleniumCheckSettings
+    def ignore(
+        self,
+        *target_path,  # type: RegionLocator
+        **kwargs  # type: Optional[CodedRegionPadding]
+    ):
+        # type: (...) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
@@ -297,7 +288,7 @@ class SeleniumCheckSettings(CheckSettings):
 
     @overload  # noqa
     def accessibility(self, target_path, type):  # noqa
-        # type:(Locator, AccessibilityRegionType) -> SeleniumCheckSettings
+        # type:(RegionLocator, AccessibilityRegionType) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
@@ -320,27 +311,27 @@ class SeleniumCheckSettings(CheckSettings):
         return super(SeleniumCheckSettings, self).accessibility(region, type)
 
     @overload  # noqa
-    def region(self, region):
+    def region(self, region):  # noqa
         # type: (Region) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
-    def region(self, css_selector):
+    def region(self, css_selector):  # noqa
         # type: (CssSelector) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
-    def region(self, target_path):
-        # type: (Locator) -> SeleniumCheckSettings
+    def region(self, target_path):  # noqa
+        # type: (RegionLocator) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
-    def region(self, element):
+    def region(self, element):  # noqa
         # type: (AnyWebElement) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
-    def region(self, by):
+    def region(self, by):  # noqa
         # type: (BySelector) -> SeleniumCheckSettings
         pass
 
@@ -349,13 +340,13 @@ class SeleniumCheckSettings(CheckSettings):
         if isinstance(region, Region):
             self.values.target_region = region
         elif is_list_or_tuple(region):
-            self.values.target_selector = TargetPath.region(*region)
-        elif isinstance(region, string_types):
-            self.values.target_selector = TargetPath.region(region)
-        elif isinstance(region, Locator):
-            self.values.target_selector = region
-        elif is_webelement(region):
-            self.values.target_element = region
+            path = self.values.target_locator or TargetPath
+            self.values.target_locator = path.region(*region)
+        elif isinstance(region, string_types) or is_webelement(region):
+            path = self.values.target_locator or TargetPath
+            self.values.target_locator = path.region(region)
+        elif isinstance(region, RegionLocator):
+            self.values.target_locator = region
         else:
             raise TypeError("region method called with argument of unknown type!")
         return self
@@ -382,7 +373,7 @@ class SeleniumCheckSettings(CheckSettings):
 
     @overload  # noqa
     def frame(self, target_path):
-        # type: (Locator) -> SeleniumCheckSettings
+        # type: (RegionLocator) -> SeleniumCheckSettings
         pass
 
     def frame(self, frame):  # noqa
@@ -393,11 +384,11 @@ class SeleniumCheckSettings(CheckSettings):
         elif isinstance(frame, string_types):
             fl.frame_name_or_id = frame
         elif is_webelement(frame):
-            fl.frame_element = frame
+            fl.frame_locator = TargetPath.frame(frame)
         elif is_list_or_tuple(frame):
-            fl.frame_selector = TargetPath.frame(*frame)
-        elif isinstance(frame, Locator):
-            fl.frame_selector = frame
+            fl.frame_locator = TargetPath.frame(*frame)
+        elif isinstance(frame, RegionLocator):
+            fl.frame_locator = frame
         else:
             raise TypeError("frame method called with argument of unknown type!")
         self.values.frame_chain.append(fl)
@@ -409,29 +400,21 @@ class SeleniumCheckSettings(CheckSettings):
         return self
 
     def _region_provider_from(self, region, method_name, padding):
-        if isinstance(region, string_types):
+        if isinstance(region, string_types) or is_webelement(region):
             return RegionBySelector(TargetPath.region(region), padding)
         elif is_list_or_tuple(region):
             return RegionBySelector(TargetPath.region(*region), padding)
-        elif isinstance(region, Locator):
+        elif isinstance(region, RegionLocator):
             return RegionBySelector(region, padding)
-        elif is_webelement(region):
-            return RegionByElement(region, padding)
         return super(SeleniumCheckSettings, self)._region_provider_from(
             region, method_name, padding
         )
 
-    def _set_scroll_root_selector(self, target_path):
+    def _set_scroll_root_locator(self, target_path):
         if len(self.values.frame_chain) == 0:
-            self.values.scroll_root_selector = target_path
+            self.values.scroll_root_locator = target_path
         else:
-            self.values.frame_chain[-1].scroll_root_selector = target_path
-
-    def _set_scroll_root_element(self, element):
-        if len(self.values.frame_chain) == 0:
-            self.values.scroll_root_element = element
-        else:
-            self.values.frame_chain[-1].scroll_root_element = element
+            self.values.frame_chain[-1].scroll_root_locator = target_path
 
     @overload  # noqa
     def scroll_root_element(self, element):
@@ -445,7 +428,7 @@ class SeleniumCheckSettings(CheckSettings):
 
     @overload  # noqa
     def scroll_root_element(self, target_path):
-        # type: (Locator) -> SeleniumCheckSettings
+        # type: (RegionLocator) -> SeleniumCheckSettings
         pass
 
     @overload  # noqa
@@ -454,35 +437,31 @@ class SeleniumCheckSettings(CheckSettings):
         pass
 
     def scroll_root_element(self, element_or_selector):  # noqa
-        if isinstance(element_or_selector, string_types):
-            self._set_scroll_root_selector(TargetPath.region(element_or_selector))
+        if isinstance(element_or_selector, string_types) or is_webelement(
+            element_or_selector
+        ):
+            self._set_scroll_root_locator(TargetPath.region(element_or_selector))
         elif is_list_or_tuple(element_or_selector):
-            self._set_scroll_root_selector(TargetPath.region(*element_or_selector))
-        elif isinstance(element_or_selector, Locator):
-            self._set_scroll_root_selector(element_or_selector)
-        elif is_webelement(element_or_selector):
-            self._set_scroll_root_element(element_or_selector)
+            self._set_scroll_root_locator(TargetPath.region(*element_or_selector))
+        elif isinstance(element_or_selector, RegionLocator):
+            self._set_scroll_root_locator(element_or_selector)
         else:
             raise TypeError("Unsupported type of scroll root element")
         return self
 
     def _floating_provider_from(self, region, bounds):
-        if is_webelement(region):
-            return FloatingRegionByElement(region, bounds)
-        elif isinstance(region, string_types):
+        if isinstance(region, string_types) or is_webelement(region):
             return FloatingRegionBySelector(TargetPath.region(region), bounds)
         elif is_list_or_tuple(region):
             return FloatingRegionBySelector(TargetPath.region(*region), bounds)
-        elif isinstance(region, Locator):
+        elif isinstance(region, RegionLocator):
             return FloatingRegionBySelector(region, bounds)
         return super(SeleniumCheckSettings, self)._floating_provider_from(
             region, bounds
         )
 
     def _accessibility_provider_from(self, region, accessibility_region_type):
-        if is_webelement(region):
-            return AccessibilityRegionByElement(region, accessibility_region_type)
-        elif isinstance(region, string_types):
+        if isinstance(region, string_types) or is_webelement(region):
             return AccessibilityRegionBySelector(
                 TargetPath.region(region), accessibility_region_type
             )
@@ -490,7 +469,7 @@ class SeleniumCheckSettings(CheckSettings):
             return AccessibilityRegionBySelector(
                 TargetPath.region(*region), accessibility_region_type
             )
-        elif isinstance(region, Locator):
+        elif isinstance(region, RegionLocator):
             return AccessibilityRegionBySelector(region, accessibility_region_type)
         return super(SeleniumCheckSettings, self)._accessibility_provider_from(
             region, accessibility_region_type
