@@ -3,14 +3,12 @@ from typing import TYPE_CHECKING, Iterable, Text, Tuple, Union
 
 import attr
 import cattr
-from selenium.webdriver.common.by import By
-from selenium.webdriver.remote.webelement import WebElement
-from six import text_type
 
 from applitools.common import (
     AccessibilityGuidelinesVersion,
     AccessibilityLevel,
     AccessibilityRegionType,
+    AndroidDeviceInfo,
 )
 from applitools.common import ChromeEmulationInfo as ApiChromeEmulationInfo
 from applitools.common import DesktopBrowserInfo, DeviceName, FloatingBounds
@@ -24,11 +22,12 @@ from applitools.common import (
     SessionType,
     StitchMode,
     TestResults,
+    TestResultsSummary,
     VisualGridOption,
 )
 from applitools.common.utils.json_utils import attr_from_json, underscore_to_camelcase
 
-from ..common.geometry import Rectangle
+from ..common.geometry import Rectangle, RectangleSize
 from ..core import (
     FloatingRegionByRectangle,
     GetRegion,
@@ -38,14 +37,9 @@ from ..core import (
 )
 from ..core.fluent import AccessibilityRegionByRectangle
 from ..core.locators import VisualLocatorSettingsValues
-from .fluent import (
-    FloatingRegionByElement,
-    FloatingRegionBySelector,
-    RegionByElement,
-    RegionBySelector,
-)
-from .fluent.region import AccessibilityRegionByElement, AccessibilityRegionBySelector
-from .fluent.target_path import FrameLocator, Locator, RegionLocator, ShadowDomLocator
+from .fluent import FloatingRegionBySelector, RegionBySelector
+from .fluent.region import AccessibilityRegionBySelector
+from .fluent.target_path import RegionLocator
 
 if TYPE_CHECKING:
     from typing import Any, Dict, List, Optional, Union
@@ -56,6 +50,7 @@ if TYPE_CHECKING:
     from applitools.common.selenium import BrowserType, Configuration
     from applitools.core.batch_close import _EnabledBatchClose
 
+    from ..common.ultrafastgrid import IRenderBrowserInfo
     from ..common.utils.custom_types import ViewPort
     from . import OCRRegion
     from .fluent import FrameLocator, SeleniumCheckSettings
@@ -178,80 +173,6 @@ class FloatingRegion(object):
         )
 
 
-@attr.s
-class TransformedElement(object):
-    element_id = attr.ib(type=text_type)
-
-    @classmethod
-    def convert(cls, web_element):
-        # type: (Optional[WebElement]) -> Optional[TransformedElement]
-        if web_element:
-            return TransformedElement(web_element._id)  # noqa
-        else:
-            return None
-
-
-@attr.s
-class TransformedSelector(object):
-    selector = attr.ib()  # type: Union[Text, TransformedSelector]
-    type = attr.ib()  # type: Optional[Text]
-    shadow = attr.ib()  # type: Union[Text, TransformedSelector]
-    frame = attr.ib()  # type: Union[Text, TransformedSelector]
-
-    @classmethod
-    def from_by(cls, is_selenium, selector, by=None, shadow=None, frame=None):
-        if is_selenium:
-            if by == By.ID:
-                by = By.CSS_SELECTOR
-                selector = '[id="{}"]'.format(selector)
-            elif by == By.TAG_NAME:
-                by = By.CSS_SELECTOR
-            elif by == By.CLASS_NAME:
-                by = By.CSS_SELECTOR
-                selector = "." + selector
-            elif by == By.NAME:
-                by = By.CSS_SELECTOR
-                selector = '[name="{}"]'.format(selector)
-        return cls(selector, by, shadow, frame)
-
-    @classmethod
-    def convert(cls, is_selenium, locator):
-        # type: (bool, Locator) -> TransformedSelector
-        if locator is None:
-            return None
-        stack = []
-        while locator:
-            stack.append(locator)
-            locator = locator.parent
-        return cls._convert_list(is_selenium, stack)
-
-    @classmethod
-    def _convert_list(cls, is_selenium, stack):
-        if stack:
-            head = stack.pop()
-            if isinstance(head, RegionLocator):
-                return cls.from_by(is_selenium, head.selector, head.by)
-            elif isinstance(head, ShadowDomLocator):
-                return cls.from_by(
-                    is_selenium,
-                    head.selector,
-                    head.by,
-                    cls._convert_list(is_selenium, stack),
-                )
-            elif isinstance(head, FrameLocator):
-                return cls.from_by(
-                    is_selenium,
-                    head.selector,
-                    head.by,
-                    frame=cls._convert_list(is_selenium, stack),
-                )
-
-            else:
-                raise TypeError("Unexpected Locator type", type(head))
-        else:
-            return None
-
-
 @attr.s
 class MatchSettingsExact(object):
     min_diff_intensity = attr.ib()  # type: int
@@ -295,6 +216,11 @@ class IosDeviceInfo(object):
     screen_orientation = attr.ib(default=None)  # type: Optional[ScreenOrientation]
 
 
+@attr.s
+class AndroidDeviceRenderer(object):
+    android_device_info = attr.ib()  # type: AndroidDeviceInfo
+
+
 @attr.s
 class IOSDeviceRenderer(object):
     ios_device_info = attr.ib()  # type: IosDeviceInfo
@@ -320,11 +246,14 @@ class DebugScreenshotHandler(object):
 
 
 LogHandler = Union[FileLogHandler, ConsoleLogHandler]
-ElementReference = Union[TransformedElement, TransformedSelector]
+ElementReference = dict
 RegionReference = Union[ElementReference, Region]
 FrameReference = Union[ElementReference, int, Text]
 BrowserInfo = Union[
-    DesktopBrowserRenderer, ChromeEmulationDeviceRenderer, IOSDeviceRenderer
+    DesktopBrowserRenderer,
+    ChromeEmulationDeviceRenderer,
+    IOSDeviceRenderer,
+    AndroidDeviceRenderer,
 ]
 
 
@@ -336,26 +265,27 @@ def record_convert(records):
         return None
 
 
-def element_reference_convert(is_selenium, selector=None, element=None):
-    # type: (bool, Optional[Locator], Optional[WebElement]) -> ElementReference
-    if selector is not None:
-        return TransformedSelector.convert(is_selenium, selector)
-    elif element is not None:
-        return TransformedElement.convert(element)
-    else:
+def optional_element_reference_convert(is_selenium, locator=None):
+    # type: (bool, Optional[RegionLocator]) -> Optional[ElementReference]
+    if locator is None:
         return None
+    else:
+        return locator.to_dict(is_selenium)
 
 
 def frame_reference_convert(
-    is_selenium, selector=None, element=None, number=None, name=None
+    is_selenium,  # type: bool
+    locator=None,  # type: Optional[RegionLocator]
+    number=None,  # type: Optional[int]
+    name=None,  # type: Optional[Text]
 ):
-    # type: (bool, Optional[List[Text, Text]], WebElement, int, Text) -> FrameReference
+    # type: (...) -> FrameReference
     if name is not None:
         return name
     elif number is not None:
         return number
     else:
-        return element_reference_convert(is_selenium, selector, element)
+        return locator.to_dict(is_selenium)
 
 
 def browsers_info_convert(browsers_info):
@@ -370,6 +300,8 @@ def browsers_info_convert(browsers_info):
                     ChromeEmulationInfo(bi.device_name, bi.screen_orientation)
                 )
             )
+        elif isinstance(bi, AndroidDeviceInfo):
+            result.append(AndroidDeviceRenderer(bi))
         elif isinstance(bi, ApiIosDeviceInfo):
             result.append(
                 IOSDeviceRenderer(
@@ -383,10 +315,8 @@ def browsers_info_convert(browsers_info):
 
 def target_reference_convert(is_selenium, values):
     # type: (bool, SeleniumCheckSettingsValues) -> Optional[RegionReference]
-    if values.target_selector:
-        return TransformedSelector.convert(is_selenium, values.target_selector)
-    elif values.target_element:
-        return TransformedElement.convert(values.target_element)
+    if values.target_locator:
+        return values.target_locator.to_dict(is_selenium)
     elif values.target_region:
         return Region.convert(values.target_region)
     else:
@@ -394,11 +324,9 @@ def target_reference_convert(is_selenium, values):
 
 
 def ocr_target_convert(is_selenium, target):
-    # type:(bool, Union[Locator, WebElement, Region]) -> RegionReference
-    if isinstance(target, Locator):
-        return TransformedSelector.convert(is_selenium, target)
-    elif isinstance(target, WebElement):
-        return TransformedElement.convert(target)
+    # type:(bool, Union[RegionLocator, WebElement, Region]) -> RegionReference
+    if isinstance(target, RegionLocator):
+        return target.to_dict(is_selenium)
     else:
         return Region.convert(target)
 
@@ -407,12 +335,8 @@ def region_references_convert(is_selenium, regions):
     # type: (bool, List[GetRegion]) -> List[RegionReference]
     results = []
     for r in regions:
-        if isinstance(r, RegionByElement):
-            results.append(TransformedElement.convert(r._element))  # noqa
-        elif isinstance(r, RegionBySelector):
-            results.append(
-                TransformedSelector.convert(is_selenium, r._target_path)  # noqa
-            )
+        if isinstance(r, RegionBySelector):
+            results.append(r._target_path.to_dict(is_selenium))  # noqa
         elif isinstance(r, RegionByRectangle):
             results.append(Region.convert(r._region))  # noqa
         else:
@@ -424,11 +348,8 @@ def floating_region_references_convert(is_selenium, regions):
     # type: (bool, List[GetRegion]) -> List[FloatingRegion]
     results = []
     for r in regions:
-        if isinstance(r, FloatingRegionByElement):
-            region = TransformedElement.convert(r._element)  # noqa
-            bounds = r._bounds  # noqa
         if isinstance(r, FloatingRegionBySelector):
-            region = TransformedSelector.convert(is_selenium, r._target_path)  # noqa
+            region = r._target_path.to_dict(is_selenium)  # noqa
             bounds = r._bounds  # noqa
         elif isinstance(r, FloatingRegionByRectangle):
             region, bounds = Region.convert(r._rect), r._bounds  # noqa
@@ -442,11 +363,8 @@ def accessibility_region_references_convert(is_selenium, regions):
     # type: (bool, List[GetRegion]) -> List[AccessibilityRegion]
     results = []
     for r in regions:
-        if isinstance(r, AccessibilityRegionByElement):
-            region = TransformedElement.convert(r._element)  # noqa
-            type_ = r._type  # noqa
         if isinstance(r, AccessibilityRegionBySelector):
-            region = TransformedSelector.convert(is_selenium, r._target_path)  # noqa
+            region = r._target_path.to_dict(is_selenium)  # noqa
             type_ = r._type  # noqa
         elif isinstance(r, AccessibilityRegionByRectangle):
             region, type_ = Region.convert(r._rect), r._type  # noqa
@@ -582,6 +500,7 @@ class EyesCheckConfig(object):
 @attr.s
 class EyesClassicConfig(object):
     wait_before_screenshots = attr.ib(default=None)  # type: Optional[float]
+    wait_before_capture = attr.ib(default=None)  # type: Optional[int]
     stitch_mode = attr.ib(default=None)  # type: Optional[StitchMode]
     hide_scrollbars = attr.ib(default=None)  # type: Optional[bool]
     hide_caret = attr.ib(default=None)  # type: Optional[bool]
@@ -666,6 +585,7 @@ class EyesConfig(
             force_full_page_screenshot=config.force_full_page_screenshot,
             # EyesClassicConfig
             wait_before_screenshots=config.wait_before_screenshots,
+            wait_before_capture=config.wait_before_capture,
             stitch_mode=config.stitch_mode,
             hide_scrollbars=config.hide_scrollbars,
             hide_caret=config.hide_caret,
@@ -695,15 +615,13 @@ class ContextReference(object):
             cls(
                 frame=frame_reference_convert(
                     is_selenium,
-                    frame_locator.frame_selector,
-                    frame_locator.frame_element,
+                    frame_locator.frame_locator,
                     frame_locator.frame_index,
                     frame_locator.frame_name_or_id,
                 ),
-                scroll_root_element=element_reference_convert(
+                scroll_root_element=optional_element_reference_convert(
                     is_selenium,
-                    frame_locator.scroll_root_selector,
-                    frame_locator.scroll_root_element,
+                    frame_locator.scroll_root_locator,
                 ),
             )
             for frame_locator in frame_locators
@@ -733,6 +651,7 @@ class CheckSettings(MatchSettings, ScreenshotSettings):
     render_id = attr.ib(default=None)  # type: Optional[Text]
     variation_group_id = attr.ib(default=None)  # type: Optional[Text]
     timeout = attr.ib(default=None)  # type: Optional[int]
+    wait_before_capture = attr.ib(default=None)  # type: Optional[int]
 
     @classmethod
     def convert(cls, is_selenium, values):
@@ -751,6 +670,7 @@ class CheckSettings(MatchSettings, ScreenshotSettings):
             render_id=None,  # TODO: verify
             variation_group_id=values.variation_group_id,
             timeout=values.timeout,
+            wait_before_capture=values.wait_before_capture,
             # MatchSettings
             exact=None,  # TODO: verify
             match_level=values.match_level,
@@ -781,8 +701,8 @@ class CheckSettings(MatchSettings, ScreenshotSettings):
             # ScreenshotSettings
             region=target_reference_convert(is_selenium, values),
             frames=ContextReference.convert(is_selenium, values.frame_chain),
-            scroll_root_element=element_reference_convert(
-                is_selenium, values.scroll_root_selector, values.scroll_root_element
+            scroll_root_element=optional_element_reference_convert(
+                is_selenium, values.scroll_root_locator
             ),
             fully=values.stitch_content,
         )
@@ -958,14 +878,55 @@ def demarshal_locate_result(results):
 
 
 def demarshal_test_results(results_dict_list, config):
-    # type: (List[dict], Optional[Configuration]) -> List[TestResults]
-    results = [attr_from_json(dumps(r), TestResults) for r in results_dict_list]
-    if config:
-        for result in results:
-            if result:  # in case of internal USDK failure, None result is observed
-                result.set_connection_config(
-                    config.server_url, config.api_key, config.proxy
-                )
+    # type: (List[dict], Configuration) -> List[TestResults]
+    # in case of internal USDK failure, None result is observed
+    results = (attr_from_json(dumps(r), TestResults) for r in results_dict_list)
+    results = [r for r in results if r]
+    for result in results:
+        result.set_connection_config(config.server_url, config.api_key, config.proxy)
+    return results
+
+
+def demarshal_browser_info(browser_info_dict):
+    # type: (dict) -> Optional[IRenderBrowserInfo]
+    if browser_info_dict is None:
+        return None
+    elif "iosDeviceInfo" in browser_info_dict:
+        ios_device = browser_info_dict["iosDeviceInfo"]
+        return ApiIosDeviceInfo(
+            ios_device["deviceName"],
+            ios_device["screenOrientation"],
+            ios_device.get("iosVersion"),
+        )
+    elif "androidDeviceInfo" in browser_info_dict:
+        android_device = browser_info_dict["androidDeviceInfo"]
+        return AndroidDeviceInfo(
+            android_device["deviceName"],
+            android_device.get("screenOrientation"),
+            android_device.get("androidVersion"),
+        )
+    elif "chromeEmulationInfo" in browser_info_dict:
+        emulated_device = browser_info_dict["chromeEmulationInfo"]
+        return ApiChromeEmulationInfo(
+            emulated_device["deviceName"], emulated_device["screenOrientation"]
+        )
+    else:
+        desktop_browser = browser_info_dict.copy()
+        browser_type = desktop_browser.pop("name", None)
+        if browser_type:
+            return DesktopBrowserInfo(browser_type=browser_type, **desktop_browser)
+        else:
+            return DesktopBrowserInfo(**desktop_browser)
+
+
+def demarshal_close_manager_results(close_manager_result_dict, config):
+    results = attr_from_json(dumps(close_manager_result_dict), TestResultsSummary)
+    for converted_res, raw_res in zip(results, close_manager_result_dict["results"]):
+        converted_res.browser_info = demarshal_browser_info(raw_res.get("browserInfo"))
+        if converted_res.test_results:
+            converted_res.test_results.set_connection_config(
+                config.server_url, config.api_key, config.proxy
+            )
     return results
 
 
